<?php

namespace FiloBlu\Refilo\Model\Indexer\Core;

use FiloBlu\Refilo\Model\Indexer\Core\Provider\QueryProviderInterface;
use Magento\Framework\App\ResourceConnection;
use PDO;

/**
 * Class AbstractSqlToMongoAdapter
 * @package FiloBlu\Refilo\Model\Indexer\Core
 */
abstract class AbstractSqlToMongoAdapter implements SqlToMongoAdapterInterface
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * AbstractSqlToMongoAdapter constructor.
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(
        ResourceConnection $resourceConnection
    ) {
        $this->resourceConnection = $resourceConnection;
    }

    /**
     * @param QueryProviderInterface $provider
     * @return array
     */
    public function adaptFromSql(QueryProviderInterface $provider): array
    {
        /** @var PDO $pdo */
        $pdo = $this->resourceConnection
            ->getConnection()
            ->getConnection();

        $stmt = $pdo->query($provider->getQuery());
        $stmt->setFetchMode(\PDO::FETCH_CLASS, $this->getMongoModel());
        return $stmt->fetchAll();
    }

    /**
     * @return string
     */
    abstract public function getMongoModel(): string;
}
